package com.huaching.parking.local.parking_local_general.service.local.base.chargeCalculator;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.TypeReference;
import com.huaching.parking.local.parking_local_general.domain.local.po.ThsChargingRulesPO;
import com.huaching.parking.local.parking_local_general.domain.local.pojo.chargeCalculator.BaseChargeDetailPOJO;
import com.huaching.parking.local.parking_local_general.domain.local.pojo.chargeCalculator.OneTimeChargeDetailPOJO;
import com.huaching.parking.local.parking_local_general.enums.local.RequestChargeTypeEnums;
import com.huaching.parking.local.parking_local_general.util.local.DateUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;

import java.util.Date;

/**
 * @program: parking_local_general
 * @description: 按次收费
 * @author: lidongsheng
 * @createData: 2020-05-19 17:11
 * @updateAuthor: lidongsheng
 * @updateData: 2020-05-19 17:11
 * @updateContent: 按次收费
 * @Version: 1.0.0
 * @email: lidongshenglife@163.com
 * ************************************************
 * Copyright @ 李东升 2020. All rights reserved
 * ************************************************
 */

/*
首时段：2小时10元
后续：收费20元
{
    "fastTime": 2,
    "frameType": 1,
    "fastMoney": 100,
    "data": {
        //金额
        "money": 200,
    }
}
 */

@Component
public class ThsOneTimePayableMoney implements ThsRequestChargeAbstract {

    private static final Logger logger = LoggerFactory.getLogger(ThsOneTimePayableMoney.class);

    @Override
    public Integer getPayableMoney(ThsChargingRulesPO thsChargingRulesPO, Date inTime, Date endTime) {
        String chargeDetails = thsChargingRulesPO.getChargeDetails();
        BaseChargeDetailPOJO<OneTimeChargeDetailPOJO> detail =
                JSON.parseObject(chargeDetails, new TypeReference<BaseChargeDetailPOJO<OneTimeChargeDetailPOJO>>() {
                });
        inTime = DateUtils.getNextMinute(inTime, thsChargingRulesPO.getFreeTime());
        //1. 判断是否在免费时长内
        if (inTime.after(endTime)) {
            return 0;
        }
        long sumMoney = detail.getFastMoney();
        //2. 判断是否在首时段内
        if (detail.validateFirstTime(detail, inTime, endTime)) {
            return Math.toIntExact(sumMoney);
        }
        return detail.getData().getMoney();
    }

    @Override
    public Integer getType() {
        return RequestChargeTypeEnums.ONE_TIME.getCode();
    }

    public static void main(String[] args) {
        ThsChargingRulesPO thsChargingRulesPO = new ThsChargingRulesPO();
        thsChargingRulesPO.setFreeTime(10);
        thsChargingRulesPO.setChargeType(RequestChargeTypeEnums.ONE_TIME.getCode());
        thsChargingRulesPO.setMaxOneTimeMoney(20000000);
        thsChargingRulesPO.setChargeDetails("{\n" +
                "    \"fastTime\": 0,\n" +
                "    \"frameType\": 0,\n" +
                "    \"fastMoney\": 0,\n" +
                "    \"data\": {\n" +
                "        \"money\": 100\n" +
                "    }\n" +
                "}");
        ThsOneTimePayableMoney payableMoney = new ThsOneTimePayableMoney();
        Integer money = payableMoney.getPayableMoney(thsChargingRulesPO
                , DateUtils.stringConvertDate("2020-05-27 06:25:20", 3)
                , DateUtils.stringConvertDate("2020-05-27 15:20:20", 3));
        System.out.println(money);
    }
}
